# LibBlog.py is part of The Ming Blog Server
# billy_bob_ming@yahoo.com

# Copyright (C) 2005, 2006  Richard Harris, TinPenguin
# Released under the GNU General Public License
# (See the included COPYING file)

# The Ming Blog Server is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# The Ming Blog Server is distributed in the hope that they will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with The Ming Blog Server; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import cgi,os,re
from Article import Article
from stat import S_ISDIR,ST_MODE
from dircache import listdir

class LibBlog:

#	BEGIN = "<div id=\"permcontent\">\n<h2>Archives</h2>\n<h3>By $TYPE</h3>\n<ul>\n"
#	END = "\n</ul>\n</div>\n"
	CGI = "cgibin"
	DATA = cgi.FieldStorage()
	if os.path.exists(os.path.expanduser("~/.my_machine")):
		DIR = os.path.expanduser("~/htdocs/a_htdocs/blogs")
	else:
		DIR = os.path.expanduser("~/public_html/blogs")
	N = 8
	MONTHS = {
		"01":"January",
		"02":"February",
		"03":"March",
		"04":"April",
		"05":"May",
		"06":"June",
		"07":"July",
		"08":"August",
		"09":"September",
		"10":"October",
		"11":"November",
		"12":"December",
		}

	def __surNLine(self, text):
		"""For surrounding text with newlines"""
		return "\n"+text+"\n"

	def read_lines(self, fname):
		try:
			f = open(fname)
			lines = f.readlines()
			f.close()
		except:
			lines = []
		return lines

	def content(self, dir, author, email):
		"""Gets information from each entry using Article.py"""
		if self.DATA.has_key('article'): # For viewing one specific entry
			article = Article(self, os.path.join(self.DIR,dir), author, email)
			a = article.get_article(self.DATA['article'].value)
			return a
		else: # For viewing all entries (default)
			c = []
			files = listdir(os.path.join(self.DIR,dir,'entries'))
			files.sort()
			if len(files) > self.N: files = files[-self.N:]
			files.reverse()
			article = Article(self, os.path.join(self.DIR,dir), author, email)
			for file in files:
				a = article.get_article(file)
				c.append(a)
			return ''.join(c)

	def do_page(self,dir,title,tagline,intro,footer,links,content):
		"""Puts everything into the HTML template, blog.html"""
		if self.DATA.has_key('article'): # For viewing one specific entry
			intro = ""
		lines = self.read_lines(os.path.join(self.DIR,dir,'blog.html'))
		sfooter = self.read_lines(os.path.join(self.DIR,'site.footer'))
		if not (sfooter == []):
			try:
				lines = lines[:lines.index("<!--site footer-->\n")]+sfooter+\
						lines[lines.index("<!--site footer-->\n"):]
			except: pass				# no footer marker in html
		new = []
		head = 1
		for line in lines:
			if "</head" in line:
				head = 0
			elif head:
				line = re.sub("\$DIR",dir,line)
				line = re.sub("\$TITLE",title,line)
			if "<div" in line: # Put right information in each <div>
				if '"title"' in line: line = self.__surNLine(line+title)
				if '"tagline"' in line: line = self.__surNLine(line+tagline)
				if '"intro"' in line: line = self.__surNLine(line+intro)
				if '"footer"' in line: line = self.__surNLine(line+footer)
				if '"links"' in line: line = self.__surNLine(line+links)
				if '"content"' in line: line = self.__surNLine(line+content)
			new.append(line)
		return ''.join(new)

	def get_date(self, article):
		"""Formats date based on article filename"""
		str = article[0:8]
		yr = str[:4]
		mo = str[4:6]
		mo = self.MONTHS[mo]
		dy = str[6:8]
		return mo+" "+dy+", "+yr

	def get_dir(self):
		"""Finds out the blog wanted, and sets the directory accordingly"""
		if not self.DATA.has_key('blog'): # Probably reached here by error
			return "none"
		dir = self.DATA['blog'].value
		if not S_ISDIR(os.stat(os.path.join(self.DIR,dir))[ST_MODE]): # Ditto
			return "no_dir"
		else:
			return dir

	def get_lines(self,dir,name):
		"""Reads lines from any given file starting with 'blog.' """
		lines = self.read_lines(os.path.join(self.DIR,dir,'blog.'+name))
		if (name == "links"):				# pull /blogs/site.links
			lines = lines + self.read_lines(os.path.join(self.DIR,'site.links'))
		return ''.join(lines)

	def get_text(self,dir):
		"""Gets general blog information from blog.txt configuration file"""
		lines = self.read_lines(os.path.join(self.DIR,dir,'blog.txt'))
		# Grabs each part of file, using colon seperated values
		author = lines[0].split(":")[1].strip()
		email = lines[1].split(":")[1].strip()
		title = lines[2].split(":")[1].strip()
		tagline = lines[3].split(":")[1].strip()
		return author, email, title, tagline

	def give_headers(self):
		print "Content-type: text/html\n\n"

